<?php

declare(strict_types=1);

namespace mirzaev\marina\controllers;

// Файлы проекта
use mirzaev\marina\controllers\core;

// Discord framework
use Discord\Discord as discord,
	Discord\Parts\Channel\Message as message,
	Discord\Parts\Guild\Role as role,
	Discord\Parts\User\Member as member,
	Discord\WebSockets\Event as event,
	Discord\Builders\MessageBuilder as _message;

// Встроенные библиотеки
use exception;

/**
 * Контроллер головного управления
 *
 * @package mirzaev\marina\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class index extends core
{
	/**
	 * Главная страница
	 *
	 * @param array $parameters Параметры запроса
	 *
	 * @return void
	 */
	public function index(array $parameters = []): void
	{
		$this->discord->on('init', function (discord $discord) {
			// Запись в буфер вывода (терминал)
			echo "Marina is ready!", PHP_EOL;

			// Сообщение: "марина"
			$discord->on(event::MESSAGE_CREATE, function (message $message, discord $discord) {
				if ($message->author->bot) {
					// Do nothing
					return;
				}

				// Запись в буфер вывода (терминал)
				echo "{$message->author->username}: {$message->content}", PHP_EOL;

				if (mb_stristr($message->content, 'марина') !== false) {
					$message->reply(_message::new()->setContent('ЗДАРОВА'));
				}
			});

			// Пользователь: обновление роли
			$discord->on(event::GUILD_MEMBER_UPDATE, function (member $new, discord $discord, ?member $old = null) {
				// Запись в буфер вывода (терминал)

				// Инициализация реестра ролей пользователя
				$_roles = array_reverse($new->roles->toArray());

				/**
				 *
				 *  НАЧАЛО БЛОКА: "если ни одна роль не назначена, то назначить другие роли"
				 *
				 */

				// Инициализация реестров ролей
				$roles = require '../settings/discord/roles/xor.php';

				// Инициализация статуса о том, что ни одна из ролей в реестре необходимых ролей не назначена
				$fail = true;

				// Проверка на назначение хотя бы одной роли из реестра необходимых ролей
				foreach ($_roles as $role)
					if (in_array($role->id, $roles['required'], true)) $fail = false;

				// Если ни одна роль из реестра не назначена, то выдать роли из реестра ролей назначения, иначе удалить роли из реестра ролей назначения
				if ($fail) foreach ($roles['else'] as $role => $reason) $new->addRole($role, $reason);
				else foreach ($roles['else'] as $role => $reason) $new->removeRole($role, 'CONDITION NOT COMPLIED: ' . $reason);

				// Деинициализация неактуальных переменных
				unset($roles, $fail);

				/**
				 *
				 * КОНЕЦ БЛОКА: "если ни одна роль не назначена, то назначить другие роли"
				 *
				 */

				/**
				 *
				 *  НАЧАЛО БЛОКА: "если есть дубликаты ролей, то назначить доминирующую, если нет других доминирующих"
				 *
				 */

				// Инициализация реестра ролей
				$roles = require '../settings/discord/roles/xor_dublicates.php';

				// Инициализация реестра найденных основных специальностей
				$mains = [];

				// Инициализация буфера идентификатора основной роли (после обработки)
				$id;

				// Поиск основной специальности
				foreach ($_roles as $role)
					if (in_array($role->id, $roles, true)) $mains[] = $role;

				// Подсчёт количества найденных основных специальностей
				$amount = count($mains);

				if ($amount > 1) {
					// Найдено несколько основных специальностей

					// Если не передано старое состояние аккаунта, то пропустить проверку старого состояния
					if (is_null($old)) goto deduplication;

					// Поиск добавленных ролей
					$diff = array_values(array_diff($_roles, $old->roles->toArray()));

					if (count($diff) === 1 && in_array($diff[0]->id, $roles, true)) {
						// Добавлена одна роль и она является доминирующей

						// Удаление всех основных ролей кроме добавленной
						foreach ($mains as $role)
							if ($role->id === $diff[0]->id) continue;
							else $new->removeRole($role);
					} else {
						// Добавлено несколько ролей, либо добавлена одна роль, но она не является доминирующей

						deduplication:

						for ($i = $amount - 1; $i > 0; --$i) {
							// Перебор реестра найденных основных ролей

							// Удаление основной роли (будут удалены все кроме первой в списке, то есть самой приоритетной в discord)
							$new->removeRole($mains[$i]);

							// Инициализация основной роли
							$id = $mains[0]->id;
						}
					}
				} else if ($amount === 1) $id = $mains[0]->id;
				else if ($amount === 0) {
					// Не найдена основная специальность

					foreach ($_roles as $role) {
						// Перебор ролей

						if (array_key_exists($role->id, $roles)) {
							// Найдена вторичная специальность

							// Замена вторичной специальности основной специальностью
							$new->removeRole($role->id, 'Main specialty chosen');
							$new->addRole($roles[$role->id], 'Main specialty chosen');

							// Реинициализация основной роли
							$id = $roles[$role->id];

							// Завершение выполнения (успех)
							break;
						}
					}
				}

				if (isset($id)) {
					// Найден идентификатор основной роли

					// Инициализация дубликата основной роли (вторичная роль)
					$secondary = array_flip($roles)[$id];

					foreach ($_roles as $role) {
						// Перебор ролей

						if ($role->id === (string) $secondary) {
							// Найдена вторичная специальность дублирующая основную

							// Замена вторичной специальности основной специальностью
							$new->removeRole($secondary, 'Main specialty chosen');

							// Завершение выполнения (успех)
							break;
						}
					}
				}

				// Деиницализация неактуальных перменных
				unset($roles, $mains, $id, $amount);

				/**
				 *
				 * КОНЕЦ БЛОКА: "если ни одна роль не назначена, то назначить другие роли"
				 *
				 */
			});
		});

		$this->discord->run();
	}
}
